@@ -7,11 +7,9 @@ use Sub::Quote;
 use B 'perlstring';
 BEGIN {
   our $CAN_HAZ_XS =
-    !$ENV{MOO_XS_DISABLE}
-      &&
     _maybe_load_module('Class::XSAccessor')
       &&
-    (eval { Class::XSAccessor->VERSION('1.07') })
+    (Class::XSAccessor->VERSION > 1.06)
   ;
 }
 
@@ -23,25 +21,13 @@ sub generate_method {
   } elsif ($is eq 'rw') {
     $spec->{accessor} = $name unless exists $spec->{accessor};
   } elsif ($is eq 'lazy') {
+    $spec->{init_arg} = undef unless exists $spec->{init_arg};
     $spec->{reader} = $name unless exists $spec->{reader};
     $spec->{lazy} = 1;
     $spec->{builder} ||= '_build_'.$name unless $spec->{default};
-  } elsif ($is eq 'rwp') {
-    $spec->{reader} = $name unless exists $spec->{reader};
-    $spec->{writer} = "_set_${name}" unless exists $spec->{writer};
   } elsif ($is ne 'bare') {
     die "Unknown is ${is}";
   }
-  $spec->{builder} = '_build_'.$name if ($spec->{builder}||0) eq 1;
-  if (($spec->{predicate}||0) eq 1) {
-    $spec->{predicate} = $name =~ /^_/ ? "_has${name}" : "has_${name}";
-  }
-  if (($spec->{clearer}||0) eq 1) {
-    $spec->{clearer} = $name =~ /^_/ ? "_clear${name}" : "clear_${name}";
-  }
-  if (($spec->{trigger}||0) eq 1) {
-    $spec->{trigger} = quote_sub('shift->_trigger_'.$name.'(@_)');
-  }
   my %methods;
   if (my $reader = $spec->{reader}) {
     if (our $CAN_HAZ_XS && $self->is_simple_get($name, $spec)) {
@@ -52,8 +38,7 @@ sub generate_method {
       $self->{captures} = {};
       $methods{$reader} =
         quote_sub "${into}::${reader}"
-          => '    die "'.$reader.' is a read-only accessor" if @_ > 1;'."\n"
-             .$self->_generate_get($name, $spec)
+          => $self->_generate_get($name, $spec)
           => delete $self->{captures}
         ;
     }
@@ -145,7 +130,7 @@ sub is_simple_attribute {
   # clearer doesn't have to be listed because it doesn't
   # affect whether defined/exists makes a difference
   !grep $spec->{$_},
-    qw(lazy default builder coerce isa trigger predicate weak_ref);
+    qw(lazy default builder isa trigger predicate weak_ref);
 }
 
 sub is_simple_get {
@@ -155,12 +140,7 @@ sub is_simple_get {
 
 sub is_simple_set {
   my ($self, $name, $spec) = @_;
-  !grep $spec->{$_}, qw(coerce isa trigger weak_ref);
-}
-
-sub has_eager_default {
-  my ($self, $name, $spec) = @_;
-  (!$spec->{lazy} and ($spec->{default} or $spec->{builder}));
+  !grep $spec->{$_}, qw(isa trigger weak_ref);
 }
 
 sub _generate_get {
@@ -190,15 +170,8 @@ sub generate_get_default {
 
 sub _generate_use_default {
   my ($self, $me, $name, $spec, $test) = @_;
-  my $get_value = $self->_generate_get_default($me, $name, $spec);
-  if ($spec->{coerce}) {
-    $get_value = $self->_generate_coerce(
-      $name, $get_value,
-      $spec->{coerce}
-    )
-  }
   $self->_generate_simple_set(
-    $me, $name, $spec, $get_value
+    $me, $name, $spec, $self->_generate_get_default($me, $name, $spec)
   ).' unless '.$test;
 }
 
@@ -225,14 +198,9 @@ sub _generate_set {
   if ($self->is_simple_set($name, $spec)) {
     $self->_generate_simple_set('$_[0]', $name, $spec, '$_[1]');
   } else {
-    my ($coerce, $trigger, $isa_check) = @{$spec}{qw(coerce trigger isa)};
+    my ($trigger, $isa_check) = @{$spec}{qw(trigger isa)};
     my $simple = $self->_generate_simple_set('$self', $name, $spec, '$value');
     my $code = "do { my (\$self, \$value) = \@_;\n";
-    if ($coerce) {
-      $code .=
-        "        \$value = "
-        .$self->_generate_coerce($name, '$value', $coerce).";\n";
-    }
     if ($isa_check) {
       $code .= 
         "        ".$self->_generate_isa_check($name, '$value', $isa_check).";\n";
@@ -249,19 +217,7 @@ sub _generate_set {
     $code;
   }
 }
-
-sub generate_coerce {
-  my $self = shift;
-  $self->{captures} = {};
-  my $code = $self->_generate_coerce(@_);
-  ($code, delete $self->{captures});
-}
-
-sub _generate_coerce {
-  my ($self, $name, $value, $coerce) = @_;
-  $self->_generate_call_code($name, 'coerce', "${value}", $coerce);
-}
- 
+  
 sub generate_trigger {
   my $self = shift;
   $self->{captures} = {};
@@ -316,23 +272,15 @@ sub generate_populate_set {
 
 sub _generate_populate_set {
   my ($self, $me, $name, $spec, $source, $test) = @_;
-  if ($self->has_eager_default($name, $spec)) {
+  if (!$spec->{lazy} and
+        ($spec->{default} or $spec->{builder})) {
     my $get_indent = ' ' x ($spec->{isa} ? 6 : 4);
-    my $get_default = $self->_generate_get_default(
-                        '$new', $_, $spec
-                      );
     my $get_value = 
-      defined($spec->{init_arg})
-        ? "(\n${get_indent}  ${test}\n${get_indent}   ? ${source}\n${get_indent}   : "
-            .$get_default
-            ."\n${get_indent})"
-        : $get_default;
-    if ($spec->{coerce}) {
-      $get_value = $self->_generate_coerce(
-        $name, $get_value,
-        $spec->{coerce}
-      )
-    }
+      "(\n${get_indent}  ${test}\n${get_indent}   ? ${source}\n${get_indent}   : "
+        .$self->_generate_get_default(
+          '$new', $_, $spec
+        )
+        ."\n${get_indent})";
     ($spec->{isa}
       ? "    {\n      my \$value = ".$get_value.";\n      "
         .$self->_generate_isa_check(
@@ -352,14 +300,6 @@ sub _generate_populate_set {
     );
   } else {
     "    if (${test}) {\n"
-      .($spec->{coerce}
-        ? "      $source = "
-          .$self->_generate_coerce(
-            $name, $source,
-            $spec->{coerce}
-          ).";\n"
-        : ""
-      )
       .($spec->{isa}
         ? "      "
           .$self->_generate_isa_check(
@@ -389,33 +329,9 @@ sub _generate_simple_set {
   my ($self, $me, $name, $spec, $value) = @_;
   my $name_str = perlstring $name;
   my $simple = "${me}->{${name_str}} = ${value}";
-
   if ($spec->{weak_ref}) {
     require Scalar::Util;
-
-    # Perl < 5.8.3 can't weaken refs to readonly vars
-    # (e.g. string constants). This *can* be solved by:
-    #
-    #Internals::SetReadWrite($foo);
-    #Scalar::Util::weaken ($foo);
-    #Internals::SetReadOnly($foo);
-    #
-    # but requires XS and is just too damn crazy
-    # so simply throw a better exception
-    Moo::_Utils::lt_5_8_3() ? <<"EOC" : "Scalar::Util::weaken(${simple})";
-
-      eval { Scalar::Util::weaken($simple); 1 } or do {
-        if( \$@ =~ /Modification of a read-only value attempted/) {
-          require Carp;
-          Carp::croak( sprintf (
-            'Reference to readonly value in "%s" can not be weakened on Perl < 5.8.3',
-            $name_str,
-          ) );
-        } else {
-          die \$@;
-        }
-      };
-EOC
+    "Scalar::Util::weaken(${simple})";
   } else {
     $simple;
   }
